@@ -1,13 +1,38 @@
-use crossterm::{
-    event::{Event, KeyCode, KeyEvent, KeyModifiers, MouseEvent, MouseEventKind},
-    terminal,
-};
+//! Provides the [`InputHandler`] trait, which can be used
+//! to customize the default keybindings of minus
+
+use crossterm::event::{Event, KeyCode, KeyEvent, KeyModifiers, MouseEvent, MouseEventKind};
 
-use super::utils::InputEvent;
 #[cfg(feature = "search")]
-use super::utils::SearchMode;
+use crate::search::SearchMode;
 use crate::LineNumbers;
 
+/// Events handled by the `minus` pager.
+#[derive(Debug, Copy, Clone, PartialEq)]
+#[allow(clippy::module_name_repetitions)]
+pub enum InputEvent {
+    /// `Ctrl+C` or `Q`, exits the application.
+    Exit,
+    /// The terminal was resized. Contains the new number of rows.
+    UpdateTermArea(usize, usize),
+    /// `Up` or `Down` was pressed. Contains the new value for the upper mark.
+    /// Also sent by `g` or `G`, which behave like Vim: jump to top or bottom.
+    UpdateUpperMark(usize),
+    /// `Ctrl+L`, inverts the line number display. Contains the new value.
+    UpdateLineNumber(LineNumbers),
+    /// Restore the original prompt
+    RestorePrompt,
+    /// `/`, Searching for certain pattern of text
+    #[cfg(feature = "search")]
+    Search(SearchMode),
+    /// Get to the next match in forward mode
+    #[cfg(feature = "search")]
+    NextMatch,
+    /// Get to the previous match in forward mode
+    #[cfg(feature = "search")]
+    PrevMatch,
+}
+
 /// Define custom keybindings
 ///
 /// This trait can help define custom keybindings in case
@@ -20,19 +45,20 @@ use crate::LineNumbers;
 ///
 /// # Example
 /// ```
-/// use minus::{InputEvent, InputHandler, LineNumbers, Pager};
+/// use minus::{input::{InputEvent, InputClassifier}, LineNumbers, Pager};
 #[cfg_attr(feature = "search", doc = "use minus::SearchMode;")]
 /// use crossterm::event::{Event, KeyEvent, KeyCode, KeyModifiers};
 ///
 /// struct CustomInputHandler;
-/// impl InputHandler for CustomInputHandler {
-///     fn handle_input(
+/// impl InputClassifier for CustomInputHandler {
+///     fn classify_input(
 ///         &self,
 ///         ev: Event,
 ///         upper_mark: usize,
-///         // A `search_mode` parameter is available, if `search` feature is enabled
+///         // This `search_mode` parameter is available, only if `search` feature is enabled
 #[cfg_attr(feature = "search", doc = "        search_mode: SearchMode,")]
 ///         ln: LineNumbers,
+///         message: bool,
 ///         rows: usize
 ///     ) -> Option<InputEvent> {
 ///             match ev {
@@ -50,18 +76,20 @@ use crate::LineNumbers;
 ///     }
 /// }
 ///
-/// let pager = Pager::new().set_input_handler(
+/// let mut pager = Pager::new().unwrap();
+/// pager.set_input_handler(
 ///                 Box::new(CustomInputHandler)
 ///             );
 /// ```
 #[allow(clippy::module_name_repetitions)]
-pub trait InputHandler {
-    fn handle_input(
+pub trait InputClassifier {
+    fn classify_input(
         &self,
         ev: Event,
         upper_mark: usize,
         #[cfg(feature = "search")] search_mode: SearchMode,
         ln: LineNumbers,
+        message: bool,
         rows: usize,
     ) -> Option<InputEvent>;
 }
@@ -70,15 +98,18 @@ pub trait InputHandler {
 /// making a custom input handler struct and implementing the [`InputHandler`] trait
 pub struct DefaultInputHandler;
 
-impl InputHandler for DefaultInputHandler {
-    fn handle_input(
+impl InputClassifier for DefaultInputHandler {
+    #[allow(clippy::too_many_lines)]
+    fn classify_input(
         &self,
         ev: Event,
         upper_mark: usize,
         #[cfg(feature = "search")] search_mode: SearchMode,
         ln: LineNumbers,
+        message: bool,
         rows: usize,
     ) -> Option<InputEvent> {
+        #[allow(clippy::unnested_or_patterns)]
         match ev {
             // Scroll up by one.
             Event::Key(KeyEvent {
@@ -96,12 +127,23 @@ impl InputHandler for DefaultInputHandler {
                 Some(InputEvent::UpdateUpperMark(upper_mark.saturating_add(1)))
             }
 
+            Event::Key(KeyEvent {
+                code: KeyCode::Enter,
+                modifiers: KeyModifiers::NONE,
+            }) => {
+                if message {
+                    Some(InputEvent::RestorePrompt)
+                } else {
+                    Some(InputEvent::UpdateUpperMark(upper_mark.saturating_add(1)))
+                }
+            }
+
             // Scroll up by half screen height.
             Event::Key(KeyEvent {
                 code: KeyCode::Char('u'),
                 modifiers,
             }) if modifiers == KeyModifiers::CONTROL || modifiers == KeyModifiers::NONE => {
-                let half_screen = (terminal::size().ok()?.1 / 2) as usize;
+                let half_screen = (rows / 2) as usize;
                 Some(InputEvent::UpdateUpperMark(
                     upper_mark.saturating_sub(half_screen),
                 ))
@@ -111,7 +153,7 @@ impl InputHandler for DefaultInputHandler {
                 code: KeyCode::Char('d'),
                 modifiers,
             }) if modifiers == KeyModifiers::CONTROL || modifiers == KeyModifiers::NONE => {
-                let half_screen = (terminal::size().ok()?.1 / 2) as usize;
+                let half_screen = (rows / 2) as usize;
                 Some(InputEvent::UpdateUpperMark(
                     upper_mark.saturating_add(half_screen),
                 ))
@@ -153,14 +195,16 @@ impl InputHandler for DefaultInputHandler {
                 upper_mark.saturating_sub(rows - 1),
             )),
             Event::Key(KeyEvent {
-                code: KeyCode::PageDown,
+                code: c,
                 modifiers: KeyModifiers::NONE,
-            }) => Some(InputEvent::UpdateUpperMark(
-                upper_mark.saturating_add(rows - 1),
-            )),
+            }) if c == KeyCode::PageDown || c == KeyCode::Char(' ') => Some(
+                InputEvent::UpdateUpperMark(upper_mark.saturating_add(rows - 1)),
+            ),
 
             // Resize event from the terminal.
-            Event::Resize(_, height) => Some(InputEvent::UpdateRows(height as usize)),
+            Event::Resize(cols, rows) => {
+                Some(InputEvent::UpdateTermArea(cols as usize, rows as usize))
+            }
             // Switch line number display.
             Event::Key(KeyEvent {
                 code: KeyCode::Char('l'),
@@ -211,3 +255,5 @@ impl InputHandler for DefaultInputHandler {
         }
     }
 }
+#[cfg(test)]
+mod tests;
